const Constants = require("./Constants");
const Websocket = require("./Websocket/Websocket");
const EventEmitter = require("events");
const superagent = require("superagent");
const Collection = require("./utils/Collection");
const Rest = require("./Rest");
const logger = require("./Utils/Logger");

class Client extends EventEmitter {
    constructor(options = {}) {
        super();
        
        if (!options.token) throw new Error("No token provided.");
        if (!options.cdnUrl) throw new Error("No CDN URL provided.");
        if (!options.gwUrl) throw new Error("No gateway URL provided.");
        if (!options.root) throw new Error("No API root provided.");
        
        this.token = options.token;
        this.game = options.game || null;
        this.STATUS = options.STATUS || "online";
        this.DEBUG = options.DEBUG || false;
        this.rest = new Rest(this);
        this.ws = null;
        this.isConnected = false;
        this.guilds = new Collection();
        this.users = new Collection();
        this.emojis = new Collection();
        this.channels = new Collection();
        this.self = null;
        this.afk = options.afk || false;
        this.gameType = options.gameType || 0;
        this.cdnUrl = options.cdnUrl;
        this.gwUrl = options.gwUrl;
        this.root = options.root;
    }

    connect() {
        if (this.isConnected) throw new Error("Attempted to connect while already connected.");

        try {
            this.ws = new Websocket(this);
        } catch (e) {
            logger.error(e.stack);
        }
    }

    get ping() {
        if (!this.ws) throw new Error("No websocket connection.");

        return this.ws.lastHeartbeatAck - this.ws.lastHeartbeatSent;
    }
};

module.exports = Client;