const GuildChannel = require("./GuildChannel");
const Collection = require("../Utils/Collection");
const Role = require("./Role");

class Guild {
    constructor(d, client) {
        this.raw = d;
        this.client = client;
        this.id = d.id;
        this.name = d.name;
        this.iconURL = `${client.cdnUrl}/icons/${this.id}/${d.icon}.png`;
        this.splash = d.splash;
        this.owner = d.owner || null;
        this.ownerID = d.owner_id;
        this.permissions = d.permissions || null;
        this.region = d.region;
        this.afkChannelID = d.afk_channel_id;
        this.afkTimeout = d.afk_timeout;
        this.enableEmbeds = d.embeds_enabled || null;
        this.embedChannelID = d.embed_channel_id || null;
        this.verificationLevel = d.verification_level;
        this.explicitFilter = d.explicit_content_filter || null;
        this.mfaLevel = d.mfa_level;
        this.widgetEnabled = d.widget_enabled;
        this.widgetChannelID = d.widget_channel_id || null;
        this.systemChannelID = d.system_channel_id;
        this.joinedAtTimestamp = d.joined_at;
        this.large = d.large;
        this.memberCount = d.member_count;
        this.members = new Collection();
        this.roles = new Collection();
        this.channels = new Collection();
        this.emojis = new Collection();
        this.presences = d.presences;
    }

    _initialize() {
        this.raw.members.forEach((member) => {
            this.members.set(member.user.id, member);
            this.client.users.set(member.user.id, member.user);
        });

        this.raw.channels.forEach((channel) => {
            let c = new GuildChannel(channel, this.client);
            this.channels.set(c.id, c);
            this.client.channels.set(c.id, c);
        });

        this.raw.roles.forEach((role) => {
            let r = new Role(role);
            this.roles.set(r.id, r);
        })

        this.raw.emojis.forEach((emoji) => {
            this.emojis.set(emoji.id, emoji);
            this.client.emojis.set(emoji.id, emoji);
        });
    }
};

module.exports = Guild;